@@ -29,14 +29,14 @@ from __future__ import absolute_import
 
 from .util import (
     click_force_option, click_callback, click_parse_b32_key,
-    click_postpone_execution, prompt_for_touch, UpperCaseChoice)
+    click_postpone_execution, prompt_for_touch, EnumChoice)
 from ..util import (
     TRANSPORT, generate_static_pw, modhex_decode,
     modhex_encode, parse_key, parse_b32_key)
 from binascii import a2b_hex, b2a_hex
 from .. import __version__
 from ..driver_otp import YkpersError
-from ..otp import OtpController, PrepareUploadFailed
+from ..otp import OtpController, PrepareUploadFailed, SlotConfig
 from ..scancodes import KEYBOARD_LAYOUT
 import logging
 import os
@@ -339,7 +339,9 @@ def yubiotp(ctx, slot, public_id, private_id, key, no_enter, force,
                            abort=True, err=True)
 
     try:
-        controller.program_otp(slot, key, public_id, private_id, not no_enter)
+        controller.program_otp(slot, key, public_id, private_id, SlotConfig(
+            append_cr=not no_enter
+        ))
     except YkpersError as e:
         _failed_to_write_msg(ctx, e)
 
@@ -357,8 +359,7 @@ def yubiotp(ctx, slot, public_id, private_id, key, no_enter, force,
     '-l', '--length', type=click.IntRange(1, 38),
     help='Length of generated password.')
 @click.option(
-    '-k', '--keyboard-layout', type=UpperCaseChoice(
-            [l.name for l in KEYBOARD_LAYOUT]),
+    '-k', '--keyboard-layout', type=EnumChoice(KEYBOARD_LAYOUT),
     default='MODHEX', show_default=True,
     help='Keyboard layout to use for the static password.')
 @click.option('--no-enter', is_flag=True, help="Don't send an Enter "
@@ -379,7 +380,6 @@ def static(
     """
 
     controller = ctx.obj['controller']
-    keyboard_layout = KEYBOARD_LAYOUT[keyboard_layout]
 
     if password and len(password) > 38:
         ctx.fail('Password too long (maximum length is 38 characters).')
@@ -394,8 +394,9 @@ def static(
     if not force:
         _confirm_slot_overwrite(controller, slot)
     try:
-        controller.program_static(
-            slot, password, not no_enter, keyboard_layout=keyboard_layout)
+        controller.program_static(slot, password, keyboard_layout, SlotConfig(
+            append_cr=not no_enter
+        ))
     except YkpersError as e:
         _failed_to_write_msg(ctx, e)
 
@@ -546,7 +547,9 @@ def hotp(ctx, slot, key, digits, counter, no_enter, force):
         err=True)
     try:
         controller.program_hotp(
-            slot, key, counter, int(digits) == 8, not no_enter)
+            slot, key, counter, int(digits) == 8, SlotConfig(
+                append_cr=not no_enter
+            ))
     except YkpersError as e:
         _failed_to_write_msg(ctx, e)
 
@@ -569,8 +572,11 @@ def hotp(ctx, slot, key, digits, counter, no_enter, force):
     '-p', '--pacing', type=click.Choice(['0', '20', '40', '60']),
     default='0', show_default=True, help='Throttle output speed by '
     'adding a delay (in ms) between characters emitted.')
+@click.option('--use-numeric-keypad', is_flag=True, show_default=True,
+              help='Use scancodes for numeric keypad when sending digits.'
+              ' Helps with some keyboard layouts. ')
 def settings(ctx, slot, new_access_code, delete_access_code, enter, pacing,
-             force):
+             use_numeric_keypad, force):
     """
     Update the settings for a slot.
 
@@ -605,7 +611,11 @@ def settings(ctx, slot, new_access_code, delete_access_code, enter, pacing,
         pacing = int(pacing)
 
     try:
-        controller.update_settings(slot, enter=enter, pacing=pacing)
+        controller.update_settings(slot, SlotConfig(
+            append_cr=enter,
+            pacing=pacing,
+            numeric_keypad=use_numeric_keypad
+        ))
     except YkpersError as e:
         _failed_to_write_msg(ctx, e)
 
