@@ -45,12 +45,17 @@ class AbstractDriver(object):
 
     transport = None
 
-    def __init__(self, pid):
-        self._pid = pid
+    def __init__(self, key_type, mode):
+        self._key_type = key_type
+        self._mode = mode
 
     @property
-    def pid(self):
-        return self._pid
+    def key_type(self):
+        return self._key_type
+
+    @property
+    def mode(self):
+        return self._mode
 
     def read_serial(self):
         """
@@ -72,6 +77,10 @@ class AbstractDriver(object):
         """
         return None
 
+    @property
+    def is_in_fips_mode(self):
+        raise NotImplementedError()
+
     def read_config(self):
         raise NotImplementedError()
 
